import typer
from enum import Enum
import subprocess
import re
import sys
import os
from pathlib import Path
from typing import List
from audio_silence_marks.targets import audipo
from audio_silence_marks.types import FileInfo


def cli():
    typer.run(cli_main)


class Target(str, Enum):
    audipo = "audipo"


def cli_main(
    path: Path = typer.Argument(
        ...,
        help='is a path to files. E.g: "."',
        exists=True,
        file_okay=False,
        dir_okay=True,
        writable=False,
        readable=True,
        resolve_path=False,
    ),
    glob: str = typer.Argument(
        ..., help="argument is a pattern for selecting files. E.g.: '**/*.mp3'"
    ),
    target: Target = typer.Option(
        Target.audipo, "--target", "-t", help="Target format for marks."
    ),
    noise: int = typer.Option(
        50,
        "--noise",
        "-n",
        help="Maximum volume of the noise treated as silence in -dB",
    ),
    duration: int = typer.Option(
        1, "--noise", "-d", help="Minimum length of the silent interval in seconds"
    ),
    lst: bool = typer.Option(
        False,
        "--list",
        "-l",
        help="Simply lists matched files. Useful for GLOB debugging.",
    ),
) -> None:
    """Processes audio files using FFmpeg filter silencedetect and outputs Audipo markers JSON with the list of
    spots placed in the middle of silence intervals.

    More info on using GLOBS: https://docs.python.org/3.8/library/glob.html
    """
    main(path, glob, target, noise, duration, lst)


def log(msg) -> None:
    print(msg, end="", flush=True, file=sys.stderr)


def main(path, glob, target, noise, duration, lst) -> None:
    p = Path(path)
    g = p.glob(glob)
    files_info: List[FileInfo] = []
    if lst:
        for x in g:
            if x.is_file():
                print(x.relative_to(path))
        return
    for x in g:
        if x.is_file():
            log(f"File: {x}... ")
            try:
                intervals = get_file_intervals(x, noise, duration)
                files_info.append(
                    {
                        "path": x.parent.relative_to(path),
                        "file": x.name,
                        "size": os.path.getsize(x),
                        "intervals": intervals,
                    }
                )
                log(f"processed {len(intervals)} intervals\n")
            except ValueError as err:
                log(f"(skipped) Error: {err}\n")
    if target == "audipo":
        audipo.main(files_info)


re_line = re.compile(r"^\[silencedetect")
re_parts = [
    re.compile(r"\bsilence_start: (\d+(\.\d+)?)$"),
    re.compile(r"\bsilence_end: (\d+(\.\d+)?)"),
]


def get_file_intervals(
    file: Path, noise: int = 50, duration: int = 1
) -> List[List[int]]:
    proc = subprocess.run(
        [
            "ffmpeg",
            "-i",
            file,
            "-af",
            f"silencedetect=n=-{noise}dB:d={duration}",
            "-f",
            "null",
            "-",
        ],
        check=True,
        encoding="utf-8",
        text=True,
        stderr=subprocess.PIPE,
        stdout=subprocess.PIPE,
    )
    state = 0
    intervals = []
    pair = [0, 0]
    for line in proc.stderr.splitlines():
        if re_line.match(line):
            matches = re_parts[state].findall(line)
            if matches:
                pair[state] = int(float(matches[0][0]) * 1000)
                if state == 1:
                    intervals.append(pair)
                    pair = [0, 0]
                state = 1 if state == 0 else 0
            else:
                raise ValueError(f"Parsing error at: {line}")
    return intervals



import json
from typing import List, TypedDict
from audio_silence_marks.types import FileInfo


class AudipoMark(TypedDict):
    id: int
    pos: int


class AudipoFile(TypedDict):
    fileSize: int
    filepath: str
    marklist: List[AudipoMark]


class AudipoMarks(TypedDict):
    externalStorageDirectory: str
    files: List[AudipoFile]


def output_audipo_file_mark(index: int, interval: List[int]) -> AudipoMark:
    diff = interval[1] - interval[0]
    return {
        "id": index,
        "pos": interval[1] - 500 if diff > 1000 else interval[0] + int(diff / 2),
    }


def output_audipo_file(file_info: FileInfo) -> AudipoFile:
    return {
        "fileSize": file_info["size"],
        "filepath": str(file_info["path"])
        + ("/" if file_info["path"] else "")
        + str(file_info["file"]),
        "marklist": [
            output_audipo_file_mark(i, x) for i, x in enumerate(file_info["intervals"])
        ],
    }


def main(files_info: List[FileInfo]) -> None:
    output = {
        "externalStorageDirectory": "",
        "files": [output_audipo_file(file_info) for file_info in files_info],
    }
    print(json.dumps(output))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['audio_silence_marks', 'audio_silence_marks.targets']

package_data = \
{'': ['*']}

install_requires = \
['typer>=0.3.2,<0.4.0']

entry_points = \
{'console_scripts': ['audio_silence_marks = audio_silence_marks.main:cli']}

setup_kwargs = {
    'name': 'audio-silence-marks',
    'version': '0.1.1',
    'description': 'Tool for creating silence marks for audio files',
    'long_description': '# Create silence marks for your audio files\n\nCreates lists of silent spots in audio files using **FFmpeg** with filter \n[silencedetect](https://ffmpeg.org/ffmpeg-filters.html#silencedetect).\n\nCurrently only [Audipo](https://play.google.com/store/apps/details?id=jp.ne.sakura.ccice.audipo&hl=en_US&gl=US)\nplayer marks format is supported.\n\n## Prerequisites\n\n- [Python 3.8+](https://www.python.org/)\n- [FFmpeg](https://ffmpeg.org/)\n- Linux (haven\'t tested on Windows/Mac yet)\n\n## Installation\n\n```\n$ pip install audio_silence_marks\n```\n\n## Usage\n\nRun the tool against one file or a directory tree:\n\n```\n$ audio_silence_marks . file.mp3 > marks.audipomark\n$ audio_silence_marks . \'**/*.mp3\' > marks.audipomark\n```\n\nThen upload this file to your phone\'s directory `Interal Storage/Audipo/Mark`, e.g.: \n```\n$ adb push marks.audipomark /storage/emulated/0/Audipo/Mark/\n```\n\nOpen your Audipo player, go to `Menu > Preferences` and click on `Import all marks` item.\nRestart the player.\n\n## Result\n\nExample:\n\nUnit 23             |  Unit 24\n:-------------------------:|:-------------------------:\n![image](https://user-images.githubusercontent.com/114060/99715000-5cd86780-2ab7-11eb-8707-b7235bebebf3.png)  |  ![image](https://user-images.githubusercontent.com/114060/99714622-dc196b80-2ab6-11eb-977d-cd3d58ff1786.png)\n\n## Docs\n\n```\n$ audio_silence_marks --help\n\nUsage: audio_silence_marks [OPTIONS] PATH GLOB\n\n  Processes audio files using FFmpeg filter silencedetect and outputs Audipo\n  markers JSON with the list of spots placed in the middle of silence\n  intervals.\n\n  More info on using GLOBS: https://docs.python.org/3.8/library/glob.html\n\nArguments:\n  PATH  is a path to files. E.g: "."  [required]\n  GLOB  argument is a pattern for selecting files. E.g.: \'**/*.mp3\'\n        [required]\n\n\nOptions:\n  -t, --target [audipo]           Target format for marks.  [default: audipo]\n  -n, --noise INTEGER             Maximum volume of the noise treated as\n                                  silence in -dB  [default: 50]\n\n  -d, --noise INTEGER             Minimum length of the silent interval in\n                                  seconds  [default: 1]\n\n  -l, --list                      Simply lists matched files. Useful for GLOB\n                                  debugging.  [default: False]\n\n  --install-completion [bash|zsh|fish|powershell|pwsh]\n                                  Install completion for the specified shell.\n  --show-completion [bash|zsh|fish|powershell|pwsh]\n                                  Show completion for the specified shell, to\n                                  copy it or customize the installation.\n\n  --help                          Show this message and exit.\n```\n',
    'author': 'OnkelTem',
    'author_email': 'aneganov@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/OnkelTem/audio_silence_marks',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
